@@ -7,8 +7,10 @@ from .iotcontrol.enums import (
     Color,
     Icon,
     Precision,
+    SoundName,
     Keyboard,
     TextAlignment,
+    TitlePosition,
     SliderBarType,
     DialNumberPosition,
     DialStyle,
