@@ -4,6 +4,42 @@ import ssl
 import logging
 import zmq
 import shortuuid
+from .constants import DEVICE_PUB_URL, CONNECTION_PUB_URL
+import time
+import json
+
+
+class Dash(object):
+
+    def get_state(self) -> str:
+        return ""
+
+    def get_cfg(self, page_size_x, page_size_y):
+        cfg_str = "\tCFG\t" + self.msg_type + "\t" + json.dumps(self._cfg) + "\n"
+        return cfg_str
+
+    def __init__(self, control_id, username="", servername=""):
+        self.msg_type = "DASHIO"
+        self._cfg = {}
+        self.control_id = control_id
+        self.username = username
+        self.servername = servername
+
+    @property
+    def username(self) -> str:
+        return self._cfg["userName"]
+
+    @username.setter
+    def username(self, val: str):
+        self._cfg["userName"] = val
+
+    @property
+    def servername(self) -> str:
+        return self._cfg["hostURL"]
+
+    @servername.setter
+    def servername(self, val: str):
+        self._cfg["hostURL"] = val
 
 
 class dashConnection(threading.Thread):
@@ -13,11 +49,10 @@ class dashConnection(threading.Thread):
         if rc == 0:
             self.connected = True
             self.disconnect = False
-            if self.device_list:
-                for device in self.device_list:
-                    control_topic = "{}/{}/control".format(self.username, device.device_id)
-                    self.dash_c.subscribe(control_topic, 0)
-                    self.__send_dash_announce(device)
+            for device_id in self.device_id_list:
+                control_topic = "{}/{}/control".format(self.username, device_id)
+                self.dash_c.subscribe(control_topic, 0)
+            self._send_dash_announce()
             logging.debug("connected OK")
         else:
             logging.debug("Bad connection Returned code=%s", rc)
@@ -29,7 +64,7 @@ class dashConnection(threading.Thread):
 
     def __on_message(self, client, obj, msg):
         data = str(msg.payload, "utf-8").strip()
-        logging.debug("DASH RX: %s", data)
+        logging.debug("DASH RX:\n%s", data)
         self.tx_zmq_pub.send_multipart([self.b_connection_id, b'1', msg.payload])
 
     def __on_publish(self, client, obj, mid):
@@ -42,18 +77,21 @@ class dashConnection(threading.Thread):
         logging.debug(string)
 
     def add_device(self, device):
-        if device not in self.device_list:
-            self.device_list.append(device)
-            device.add_connection(self)
+        if device.device_id not in self.device_id_list:
+            self.device_id_list.append(device.device_id)
+            device._add_connection(self)
+            device.add_control(self.dash_control)
+
+            self.rx_zmq_sub.connect(DEVICE_PUB_URL.format(id=device._zmq_pub_id))
+            self.rx_zmq_sub.setsockopt_string(zmq.SUBSCRIBE, device._zmq_pub_id)
+
             if self.connected:
                 control_topic = "{}/{}/control".format(self.username, device.device_id)
                 self.dash_c.subscribe(control_topic, 0)
-                self.__send_dash_announce(device)
+                self._send_dash_announce()
 
-    def __send_dash_announce(self, device):
-        data_topic = "{}/{}/announce".format(self.username, device.device_id)
-        data = device.device_id_str + "\tWHO\t{}\t{}\n".format(device.device_type, device.device_name_cntrl.control_id)
-        self.dash_c.publish(data_topic, data)
+    def _send_dash_announce(self):
+        self.tx_zmq_pub.send_multipart([b'COMMAND', b'1', b"send_announce"])
 
     def set_connection(self, username, password):
         self.dash_c.disconnect()
@@ -61,7 +99,16 @@ class dashConnection(threading.Thread):
         self.dash_c.username_pw_set(username, password)
         self.dash_c.connect(self.host, self.port)
 
-    def __init__(self, username="", password="", host='dash.dashio.io', port=8883, set_by_iotdashboard=False, context=None):
+    def __init__(
+        self,
+        username="",
+        password="",
+        host='dash.dashio.io',
+        port=8883,
+        use_ssl=True,
+        set_by_iotdashboard=False,
+        context=None
+    ):
         """
         Arguments:
             host {str} -- The server name of the dash host.
@@ -76,7 +123,7 @@ class dashConnection(threading.Thread):
         self.connected = False
         self.connection_id = shortuuid.uuid()
         self.b_connection_id = self.connection_id.encode('utf-8')
-        self.device_list = []
+        self.device_id_list = []
         self.LWD = "OFFLINE"
         self.running = True
         self.username = username
@@ -90,18 +137,19 @@ class dashConnection(threading.Thread):
         self.dash_c.on_disconnect = self.__on_disconnect
         # self.dash_c.on_publish = self.__on_publish
         self.dash_c.on_subscribe = self.__on_subscribe
+        self.dash_control = Dash(self.connection_id, username, host)
+        if use_ssl:
+            self.dash_c.tls_set(
+                ca_certs=None,
+                certfile=None,
+                keyfile=None,
+                cert_reqs=ssl.CERT_REQUIRED,
+                tls_version=ssl.PROTOCOL_TLSv1_2,
+                ciphers=None,
+            )
+            self.dash_c.tls_insecure_set(False)
 
-        self.dash_c.tls_set(
-            ca_certs=None,
-            certfile=None,
-            keyfile=None,
-            cert_reqs=ssl.CERT_REQUIRED,
-            tls_version=ssl.PROTOCOL_TLSv1_2,
-            ciphers=None,
-        )
-        self.dash_c.tls_insecure_set(False)
-
-        self.dash_c.on_log = self.__on_log
+        # self.dash_c.on_log = self.__on_log
         # self.dash_c.will_set(self.data_topic, self.LWD, qos=1, retain=False)
         # Connect
         if username and password:
@@ -109,6 +157,7 @@ class dashConnection(threading.Thread):
             self.dash_c.connect(host, port)
         # Start subscribe, with QoS level 0
         self.start()
+        time.sleep(1)
 
     def close(self):
         self.running = False
@@ -116,22 +165,18 @@ class dashConnection(threading.Thread):
     def run(self):
         self.dash_c.loop_start()
 
-        tx_url_internal = "inproc://TX_{}".format(self.connection_id)
-        rx_url_internal = "inproc://RX_{}".format(self.connection_id)
-
         self.tx_zmq_pub = self.context.socket(zmq.PUB)
-        self.tx_zmq_pub.bind(tx_url_internal)
+        self.tx_zmq_pub.bind(CONNECTION_PUB_URL.format(id=self.connection_id))
 
-        rx_zmq_sub = self.context.socket(zmq.SUB)
-        rx_zmq_sub.bind(rx_url_internal)
+        self.rx_zmq_sub = self.context.socket(zmq.SUB)
 
         # Subscribe on ALL, and my connection
-        rx_zmq_sub.setsockopt(zmq.SUBSCRIBE, b"ALL")
-        rx_zmq_sub.setsockopt(zmq.SUBSCRIBE, b"ANNOUNCE")
-        rx_zmq_sub.setsockopt(zmq.SUBSCRIBE, b"ALARM")
-        rx_zmq_sub.setsockopt(zmq.SUBSCRIBE, self.b_connection_id)
+        self.rx_zmq_sub.setsockopt(zmq.SUBSCRIBE, b"ALL")
+        self.rx_zmq_sub.setsockopt(zmq.SUBSCRIBE, b"ANNOUNCE")
+        self.rx_zmq_sub.setsockopt(zmq.SUBSCRIBE, b"ALARM")
+        self.rx_zmq_sub.setsockopt_string(zmq.SUBSCRIBE, self.connection_id)
         poller = zmq.Poller()
-        poller.register(rx_zmq_sub, zmq.POLLIN)
+        poller.register(self.rx_zmq_sub, zmq.POLLIN)
 
         while self.running:
             try:
@@ -139,8 +184,10 @@ class dashConnection(threading.Thread):
             except zmq.error.ContextTerminated:
                 break
 
-            if rx_zmq_sub in socks:
-                [address, id, data] = rx_zmq_sub.recv_multipart()
+            if self.rx_zmq_sub in socks:
+                [address, id, data] = self.rx_zmq_sub.recv_multipart()
+                if not data:
+                    continue
                 msg_l = data.split(b'\t')
                 try:
                     device_id = msg_l[1].decode('utf-8').strip()
@@ -148,10 +195,12 @@ class dashConnection(threading.Thread):
                     continue
                 if address == b'ALARM':
                     data_topic = "{}/{}/alarm".format(self.username, device_id)
+                elif address == b"ANNOUNCE":
+                    data_topic = "{}/{}/announce".format(self.username, device_id)
                 else:
                     data_topic = "{}/{}/data".format(self.username, device_id)
                 if self.connected:
-                    logging.debug("DASH TX: %s", data.decode('utf-8').rstrip())
+                    logging.debug("DASH TX:\n%s", data.decode('utf-8').rstrip())
                     self.dash_c.publish(data_topic, data.decode('utf-8'))
 
         if self.connected:
@@ -159,4 +208,4 @@ class dashConnection(threading.Thread):
         self.dash_c.loop_stop()
 
         self.tx_zmq_pub.close()
-        rx_zmq_sub.close()
+        self.rx_zmq_sub.close()
