@@ -2,77 +2,68 @@ import logging
 import zmq
 import threading
 import json
+import shortuuid
 
-from .iotcontrol.name import Name
 from .iotcontrol.alarm import Alarm
 from .iotcontrol.page import Page
 from .iotcontrol.event import Event
+from .constants import *
 
 class dashDevice(threading.Thread):
 
-    """Setups and manages a connection thread to iotdashboard via TCP."""
+    """Device for IoTDashboard."""
 
-    def __on_message(self, payload):
+    def _on_message(self, payload):
         data = str(payload, "utf-8").strip()
         command_array = data.split("\n")
         reply = ""
         for ca in command_array:
             try:
-                reply += self.__on_command(ca.strip())
+                reply += self._on_command(ca.strip())
             except TypeError:
                 pass
         return reply
 
-    def __on_command(self, data):
+    def _on_command(self, data):
         data_array = data.split("\t")
         rx_device_id = data_array[0]
         # logging.debug('Device RX: %s', rx_device_id)
-        reply = ""
         if rx_device_id == "WHO":
-            reply = self.device_id_str + "\tWHO\t{}\t{}\n".format(self.device_type, self.device_name_cntrl.control_id)
-            return reply
-        elif rx_device_id != self.device_id:
-            return reply
-        cntrl_type = data_array[1]
+            return self.device_id_str + "\tWHO\t{}\t{}\n".format(self.device_type, self.device_name)
+        if rx_device_id != self.device_id:
+            return ""
+        try:
+            cntrl_type = data_array[1]
+        except KeyError:
+            return ""
         if cntrl_type == "CONNECT":
-            reply = self.connect
-        elif cntrl_type == "STATUS":
-            reply = self.__make_status()
-        elif cntrl_type == "CFG":
-            reply = self.__make_cfg(data_array[2], data_array[3])
-        elif cntrl_type == "NAME":
-            if self._set_name:
-                self.device_name_cntrl.message_rx_event(data_array[2:])
-        elif cntrl_type == "WIFI":
-            if self._set_wifi:
-                self.wifi_rx_event(data_array)
-        elif cntrl_type == "DASH":
-            if self._set_dash:
-                self.dash_rx_event(data_array)
+            return self.connect
+        if cntrl_type == "STATUS":
+            return self._make_status()
+        if cntrl_type == "CFG":
+            return self._make_cfg(data_array)
+        if cntrl_type in self._device_commands_dict:
+            self._device_commands_dict[cntrl_type](data_array)
         else:
             try:
-                key = cntrl_type + "_" + data_array[2]
-            except IndexError:
-                return reply
-            try:
-                self.control_dict[key].message_rx_event(data_array)
-            except KeyError:
+                self.control_dict[cntrl_type + "_" + data_array[2]].message_rx_event(data_array)
+            except (KeyError, IndexError):
                 pass
-        return reply
+        return ""
 
-    def __make_status(self):
-        reply = ""
+    def _make_status(self):
+        reply = "\t{device_id}\tNAME\t{device_name}\n".format(device_id=self.device_id, device_name=self._device_name)
         for key in self.control_dict.keys():
             try:
                 reply += self.control_dict[key].get_state().format(device_id=self.device_id)
-            except TypeError:
+            except (TypeError, KeyError):
                 pass
         return reply
 
-    def __make_cfg(self, page_x, page_y):
+    def _make_cfg(self, data):
         reply = self.device_id_str + '\tCFG\tDVCE\t' + json.dumps(self._cfg) + "\n"
         for key in self.control_dict.keys():
-            reply += self.device_id_str + self.control_dict[key].get_cfg(page_x, page_y)
+            reply += self.device_id_str + self.control_dict[key].get_cfg(data[2], data[3])
         return reply
 
     def send_popup_message(self, title, header, message):
@@ -107,7 +98,7 @@ class dashDevice(threading.Thread):
         logging.debug("ALARM: %s", payload)
         self.tx_zmq_pub.send_multipart([b"ALARM", b'0', payload.encode('utf-8')])
 
-    def send_data(self, data: str):
+    def _send_data(self, data: str):
         """Send data.
 
         Parameters
@@ -115,12 +106,19 @@ class dashDevice(threading.Thread):
         data : str
             Data to be sent
         """
+        if not data:
+            return
         reply_send = data.format(device_id=self.device_id)
         try:
             self.tx_zmq_pub.send_multipart([b"ALL", b'0', reply_send.encode('utf-8')])
         except zmq.error.ZMQError:
             pass
 
+    def _send_announce(self):
+        payload = self.device_id_str + "\tWHO\t{}\t{}\n".format(self.device_type, self.device_name)
+        logging.debug("ANNOUNCE: %s", payload)
+        self.tx_zmq_pub.send_multipart([b"ANNOUNCE", b'0', payload.encode('utf-8')])
+
     def add_control(self, iot_control):
         """Add a control to the connection.
 
@@ -128,39 +126,37 @@ class dashDevice(threading.Thread):
         ----------
         iot_control : iotControl
         """
-        if isinstance(iot_control, Alarm):
-            iot_control.message_tx_event += self.send_alarm
-            key = iot_control.msg_type + "_" + iot_control.control_id
-            self.alarm_dict[key] = iot_control
+        if isinstance(iot_control, Page):
+            self.number_of_pages += 1
+        try:
+            if isinstance(iot_control, Alarm):
+                iot_control.message_tx_event += self.send_alarm
+            else:
+                iot_control.message_tx_event += self._send_data
+        except AttributeError:
+            pass
+        key = iot_control.msg_type + "_" + iot_control.control_id
+        self.control_dict[key] = iot_control
+
+    def _add_connection(self, connection):
+        self.rx_zmq_sub.connect(CONNECTION_PUB_URL.format(id=connection.connection_id))
+        self.rx_zmq_sub.setsockopt(zmq.SUBSCRIBE, connection.connection_id.encode('utf-8'))
+
+    def _set_devicesetup(self, control_name: str, settable: bool):
+        if settable:
+            self._device_commands_dict[control_name.upper()] = getattr(self, control_name + '_rx_event', None)
+            self._device_setup_list.append(control_name)
         else:
-            if isinstance(iot_control, Page):
-                self.number_of_pages += 1
-            iot_control.message_tx_event += self.send_data
-            key = iot_control.msg_type + "_" + iot_control.control_id
-            self.control_dict[key] = iot_control
+            try:
+                del self._device_commands_dict[control_name.upper()]
+            except KeyError:
+                pass
+            try:
+                self._device_setup_list.remove(control_name)
+            except ValueError:
+                pass
+        self._cfg["deviceSetup"] = ','.join(self._device_setup_list)
 
-    def add_connection(self, device):
-        tx_url_internal = "inproc://RX_{}".format(device.connection_id)
-        rx_url_internal = "inproc://TX_{}".format(device.connection_id)
-        self.tx_zmq_pub.connect(tx_url_internal)
-        self.rx_zmq_sub.connect(rx_url_internal)
-        self.rx_zmq_sub.setsockopt(zmq.SUBSCRIBE, device.connection_id.encode('utf-8'))
-
-    def __set_devicesetup(self):
-        device_setup = ""
-        if self._set_dash:
-            device_setup += "dashio"
-        if self._set_name:
-            if self._set_dash:
-                device_setup += ",name"
-            else:
-                device_setup += "name"
-        if self._set_wifi:
-            if self._set_dash or self._set_name:
-                device_setup += ",wifi"
-            else:
-                device_setup += "wifi"
-        self._cfg["deviceSetup"] = device_setup
 
     def __init__(self,
                  device_type,
@@ -169,29 +165,38 @@ class dashDevice(threading.Thread):
                  edit_lock=False,
                  set_name=False,
                  set_wifi=False,
-                 set_dash=False,
+                 set_dashio=False,
+                 set_tcp=False,
+                 set_mqtt=False,
                  context=None) -> None:
         threading.Thread.__init__(self, daemon=True)
 
+        self._zmq_pub_id = shortuuid.uuid()
+        self._b_zmq_pub_id = self._zmq_pub_id.encode('utf-8')
         self.context = context or zmq.Context.instance()
         self.wifi_rx_event = Event()
         self.dash_rx_event = Event()
+        self.name_rx_event = Event()
+        self.tcp_rx_event = Event()
+        self.mqtt_rx_event = Event()
         self.device_type = device_type.strip()
         self.device_id = device_id.strip()
-        self.device_name_cntrl = Name(device_name.strip())
+        self._b_device_id = self.device_id.encode('utf-8')
+        self._device_name = device_name.strip()
+        self._device_setup_list = []
+        self._device_commands_dict = {}
         self.control_dict = {}
-        self.alarm_dict = {}
         self._cfg = {}
-        self.add_control(self.device_name_cntrl)
         self.device_id_str = "\t{}".format(device_id)
         self.connect = self.device_id_str + "\tCONNECT\n"
         self.number_of_pages = 0
 
         self.edit_lock = edit_lock
-        self._set_name = set_name
-        self._set_wifi = set_wifi
-        self._set_dash = set_dash
-        self.__set_devicesetup()
+        self.set_name = set_name
+        self.set_wifi = set_wifi
+        self.set_dashio = set_dashio
+        self.set_tcp = set_tcp
+        self.set_mqtt = set_mqtt
         self.running = True
         self.start()
 
@@ -218,7 +223,7 @@ class dashDevice(threading.Thread):
     @set_name.setter
     def set_name(self, val: bool):
         self._set_name = val
-        self.__set_devicesetup()
+        self._set_devicesetup("name", val)
 
     @property
     def set_wifi(self) -> bool:
@@ -227,16 +232,43 @@ class dashDevice(threading.Thread):
     @set_wifi.setter
     def set_wifi(self, val: bool):
         self._set_wifi = val
-        self.__set_devicesetup()
+        self._set_devicesetup("wifi", val)
 
     @property
-    def set_dash(self) -> bool:
-        return self._set_dash
+    def set_dashio(self) -> bool:
+        return self._set_dashio
 
-    @set_dash.setter
-    def set_dash(self, val: bool):
-        self._set_dash = val
-        self.__set_devicesetup()
+    @set_dashio.setter
+    def set_dashio(self, val: bool):
+        self._set_dashio = val
+        self._set_devicesetup("dashio", val)
+
+    @property
+    def set_tcp(self) -> bool:
+        return self._set_tcp
+
+    @set_tcp.setter
+    def set_tcp(self, val: bool):
+        self._set_tcp = val
+        self._set_devicesetup("tcp", val)
+
+    @property
+    def set_mqtt(self) -> bool:
+        return self._set_mqtt
+
+    @set_mqtt.setter
+    def set_mqtt(self, val: bool):
+        self._set_mqtt = val
+        self._set_devicesetup("mqtt", val)
+
+    @property
+    def device_name(self) -> str:
+        return self._device_name
+
+    @device_name.setter
+    def device_name(self, val: str):
+        self._device_name = val
+        self._send_data("\t{{device_id}}\tNAME\t{name}\t".format(name=self._device_name))
 
     def close(self):
         self.running = False
@@ -245,8 +277,9 @@ class dashDevice(threading.Thread):
         # Continue the network loop, exit when an error occurs
 
         self.tx_zmq_pub = self.context.socket(zmq.PUB)
+        self.tx_zmq_pub.bind(DEVICE_PUB_URL.format(id=self._zmq_pub_id))
         self.rx_zmq_sub = self.context.socket(zmq.SUB)
-        self.rx_zmq_sub.setsockopt(zmq.SUBSCRIBE, b"")
+        self.rx_zmq_sub.setsockopt_string(zmq.SUBSCRIBE, "COMMAND")
 
         poller = zmq.Poller()
         poller.register(self.rx_zmq_sub, zmq.POLLIN)
@@ -259,7 +292,11 @@ class dashDevice(threading.Thread):
             if self.rx_zmq_sub in socks:
                 msg = self.rx_zmq_sub.recv_multipart()
                 if len(msg) == 3:
-                    reply = self.__on_message(msg[2])
+                    if msg[0] == b"COMMAND":
+                        if msg[2] == b'send_announce':
+                            self._send_announce()
+                        continue
+                    reply = self._on_message(msg[2])
                     if reply:
                         self.tx_zmq_pub.send_multipart([msg[0], msg[1], reply.encode('utf-8')])
 
