@@ -61,6 +61,28 @@ class Icon(Enum):
     SWITCH = "Switch"
 
 
+class SoundName(Enum):
+    DEFAULT = "Default"
+    QUIET = "Quiet"
+    BELL = "Bell"
+    DOORBELL1 = "Door Bell 1"
+    DOORBELL2 = "Door Bell 2"
+    ALARM = "Alarm"
+    COW = "Cow"
+    OVERSPEED = "Overspeed"
+    SADTRUMPED = "Sad Trumped"
+    SHIPHORN = "Ship Horn"
+    SQUEAKY = "Squeaky"
+    SLAP = "Slap"
+    WHIP1 = "Whip 1"
+    WHIP2 = "Whip 2"
+    PLOP = "Plop"
+    PLOPPLIPPLIP = "Plop Plip Plip"
+    SWITCH = "Switch"
+    WHOOSH = "Woosh"
+    BOING = "Boing"
+
+
 class LabelStyle(Enum):
     BASIC = "BASIC"
     GROUP = "GROUP"
@@ -129,11 +151,12 @@ class DialNumberPosition(Enum):
     OFF = "off"
     LEFT = "left"
     RIGHT = "right"
-
+    CENTER = "center"
 
 class DialStyle(Enum):
-    STD = "upright"
-    INVERTED = "inverted"
+    PIE = "pie"
+    PIEINV = "pieinv"
+    BAR = "bar"
 
 
 class GraphLineType(Enum):
