@@ -4,8 +4,7 @@ from .enums import TitlePosition
 
 class Selector(Control):
     def __init__(self, control_id, title="A Selector", title_position=TitlePosition.BOTTOM, control_position=None):
-        super().__init__("SLCTR", control_id, control_position=control_position, title_position=title_position)
-        self.title = title
+        super().__init__("SLCTR", control_id, title=title, control_position=control_position, title_position=title_position)
         self.selection_list = []
         self._position = 0
         self._cfg["selection"] = self.selection_list
